#!/usr/bin/python3

# Programme de timelapse
# Réglage de la durée de prise de vue et de l'intervalle entre photos
# Commande d'allumage de LEDs via le port USB à destination d'un Raspberry Pi PICO
# création de la vidéo à la fin du timelapse
# D'après http://blog.rareschool.com/2021/01/controlling-raspberry-pi-pico-using.html
# Documentation Picamera : https://picamera.readthedocs.io/en/release-1.13/api_camera.html
# Licence CC BY NS SA - framboise314 - 04/10/2021

from picamera import PiCamera
from os import system
import datetime
from time import sleep
import serial

# Nombre de minutes pendant lequel le programme timelapse va fonctionner
tlminutes = 62

# Intervalle entre deux prises de vue en seconde
# L'allumage des LEDs + prise de vue dure 1 seconde
secondsinterval = 4 

# nombre d'images par seconde de la vidéo créée en fin de programme
fps = 25 

# Calcul du nombre de photos à prendre
numphotos = int((tlminutes*60)/secondsinterval) 
print("Nombre de photos à prendre : ", numphotos)

dateraw= datetime.datetime.now()
datetimeformat = dateraw.strftime("%Y-%m-%d_%H:%M")
print("RPi débute les prises de vue pour votre timelapse à : " + datetimeformat)

# Class pour l'envoi des commandes au PICO
class Sender:

    def __init__(self, device='/dev/ttyACM0', baud=9600, timeout=1):
        self.serial = serial.Serial(device, baud, timeout=timeout)

    def send(self, text: str) -> bool:
        line = '%s\r\f' % text
        self.serial.write(line.encode('UTF8'))

    def close(self):
        self.serial.close()


camera = PiCamera()

# Rotation de l'image en fonction de vos conditions de prise de vue
# et de l'orientation de la caméra
# Si cette ligne n'est pas nécessaire, mettez un # au début
camera.rotation = 180

# Résolution de l'image à capturer
camera.resolution = (1920, 1080)

# Détruit toutes les photos présentes dans le dossier Pictures
# avant de démarrer les prises de vues pour le timelapse
system('rm /home/pi/Pictures/*.jpg') 

for i in range(numphotos):
    # Créer l'objet série
    s = Sender()
    
    # Allumer les LEDs
    s.send('on()')
    sleep(0.5)
    
    # Capturer une image
    print("Prise de vue ", i)
    
    # Preview de l'image pour la mise au point
    # Mettez un # au début de la ligne quand vous utilisez le programme en réel
    camera.start_preview()
    
    # Capture d'une image et enregistrement dans le dossier Pictures
    camera.capture('/home/pi/Pictures/image{0:06d}.jpg'.format(i))
    sleep(0.2)
    
    # Eteindre les LEDs
    s.send('off()')
    sleep(0.3)
    
    # Fermer la liaison série
    s.close()
    
    # Fin de Preview de l'image pour la mise au point
    # Mettez un # au début de la ligne quand vous utilisez le programme en réel
    camera.stop_preview()
    
    # Attente jusqu'à la prochaine prise de vue
    sleep(secondsinterval)
    
# Fermer la caméra
camera.close()

print("Prise de vue terminée.")
print("Merci de patienter pendant la création de la vidéo.")

# Création de la vidéo avec ffmpeg
# Plus d'informations ici : https://ffmpeg.org/ffmpeg.html
system('ffmpeg -r {} -f image2 -s 1920x1080 -nostats -loglevel 0 -pattern_type glob -i "/home/pi/Pictures/*.jpg" -vcodec libx264 -crf 25  -pix_fmt yuv420p /home/pi/Videos/{}.mp4'.format(fps, datetimeformat))
#system('rm /home/pi/Pictures/*.jpg')

print('La vidéo du timelapse est prête. Elle est disponible ici : /home/pi/Videos/{}.mp4'.format(datetimeformat))

# fin du programme
